module Caminos where

import Array

-- este algoritmo calcula los costos de los caminos de costo mínimo entre todo par de nodos de un grafo
-- el grafo g1, es el de las filminas del algoritmo de Dijsktra

-- para obtener los costos de los caminos de costo mínimo entre todo par de nodos,
-- cargar en hugs o ghci y ejecutar: assocs (costo g1)

-- este módulo utiliza algunas funciones de Haskell para manipular arreglos

-- la declaración que sigue dice que un Grafo es un arreglo bidimensional
-- indexado por pares de enteros y que cada celda tiene un entero
type Grafo = Array (Int,Int) Int

-- inf es un número suficientemente grande
inf = 2^20 :: Int

-- dado un grafo y dos índices calcula el costo del camino de menor costo
-- de i a j utilizando backtracking
costo :: Grafo -> (Int,Int) -> Int
costo g (i,j) = m n i j
              where (_,(_,n)) = bounds g
                    m 0 i j = g!(i,j)
                    m k i j = min (m (k-1) i j) (m (k-1) i k + m (k-1) k j)

-- pregunta: ¿qué es bounds?
-- respuesta: proba ejecutar: bounds g1

-- la definición de g1 dice que es una matriz bidimensional, que ambos indices van de 1 a 6
-- luego hay una lista que dice cómo se construye la matriz: al indice (1,1) asignale 0,
-- al indice (1,2) asignale 4, etc.
g1 :: Grafo
g1 = array ((1,1),(6,6)) [((1,1),0), ((1,2),4), ((1,3),1), ((1,4),10), ((1,5),inf), ((1,6),inf),
                          ((2,1),4), ((2,2),0), ((2,3),3), ((2,4),inf), ((2,5),1), ((2,6),inf),
                          ((3,1),inf), ((3,2),2), ((3,3),0), ((3,4),8), ((3,5),4), ((3,6),inf),
                          ((4,1),2), ((4,2),inf), ((4,3),2), ((4,4),0), ((4,5),inf), ((4,6),inf),
                          ((5,1),inf), ((5,2),inf), ((5,3),3), ((5,4),inf), ((5,5),0), ((5,6),3),
                          ((6,1),inf), ((6,2),inf), ((6,3),2), ((6,4),1), ((6,5),inf), ((6,6),0)]

-- costos construye una matriz con los costos de todos los caminos de menor costo de un grafo cualquiera
-- tiene los mismos indices que el grafo que recibe
-- para la celda (i,j) se calcula el costo del camino de menor costo de i a j
costos :: Grafo -> Array (Int,Int) Int
costos g = array (bounds g) [((i,j),costo g (i,j)) | (i,j) <- indices g]


